package reqif.reqif_writer;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xml.namespace.XMLNamespacePackage;
import org.eclipse.rmf.pror.presentation.headline.HeadlinePackage;
import org.eclipse.rmf.pror.reqif10.configuration.ConfigurationPackage;
import org.eclipse.rmf.reqif10.ReqIF;
import org.eclipse.rmf.reqif10.ReqIF10Package;
import org.eclipse.rmf.reqif10.datatypes.DatatypesPackage;
import org.eclipse.rmf.reqif10.xhtml.XhtmlPackage;
import org.eclipse.rmf.serialization.ReqIFResourceFactoryImpl;
import org.eclipse.rmf.serialization.ReqIFResourceSetImpl;

public class App {
	public static void main(String[] args) {
		try {
			final String destFileName = "src/test/resources/test-dest.reqif";
			Map<String, String> map = new HashMap<String, String>();
			// Valid for ProR 0.1.0
			map.put("Valid 0.1.0", "src/test/resources/test-1.reqif");
			// Bare ReqIF no tool extensions
			map.put("Bare ReqIF", "src/test/resources/TC1300_without_namespaceprefix.reqif");
			// Valid for ProR 0.2.0
			map.put("Valid 0.2.0", "src/test/resources/TC3000_E000_S10_Reference_20120412_2336_broerkens.reqif");

			ReqIF reqif;
			try {
				registerEPackageStd();
				registerEPackageProRSpecific();
				reqif = readPlease(map.get("Valid 0.1.0"));
				justWritePlease(destFileName, reqif);
			} catch (Exception e) {
				System.out.println("Throws always: " + e.getMessage());
			}

			try {
				EPackage.Registry.INSTANCE.clear();
				reqif = readPlease(map.get("Bare ReqIF"));
				justWritePlease(destFileName, reqif);
			} catch (Exception e) {
				System.out.println("Without registration it throws always: " + e.getMessage());
			}

			registerEPackageStd();
			registerEPackageProRSpecific();
			reqif = readPlease(map.get("Bare ReqIF"));
			justWritePlease(destFileName, reqif);
			System.out.println("With registration it works");

			registerEPackageStd();
			reqif = readPlease(map.get("Bare ReqIF"));
			justWritePlease(destFileName, reqif);
			System.out.println("Works even without ProR specific registrations");

			try {
				registerEPackageStd();
				reqif = readPlease(map.get("Valid 0.2.0"));
				justWritePlease(destFileName, reqif);
			} catch (Exception e) {
				System.out.println("Throws always without ProR specific registrations: " + e.getMessage());
			}
			
			registerEPackageStd();
			registerEPackageProRSpecific();
			reqif = readPlease(map.get("Valid 0.2.0"));
			justWritePlease(destFileName, reqif);
			System.out.println("Works with ProR specific registrations");

		} catch (RuntimeException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	private static void justWritePlease(final String destFileName, ReqIF reqif)
			throws IOException {

		ReqIFResourceSetImpl resourceSet = aResourceSetPlease();
		URI uri = URI.createURI(destFileName, true);
		XMLResource resource = (XMLResource) resourceSet.createResource(uri);
		resource.getContents().add(reqif);
		resource.save(null);
	}

	private static ReqIF readPlease(final String fileName) throws IOException {

		System.out.println("\nReading " + fileName + "!");

		ReqIFResourceSetImpl resourceSet = aResourceSetPlease();
		URI uri = URI.createURI(fileName, true);
		XMLResource resource = (XMLResource) resourceSet.createResource(uri);
		resource.load(null);

		EList<EObject> rootObjects = resource.getContents();

		if (rootObjects.isEmpty())
			throw new RuntimeException("Oops! Empty!");

		ReqIF reqif = (ReqIF) rootObjects.get(0);

		return reqif;
	}

	private static ReqIFResourceSetImpl aResourceSetPlease() {

		ReqIFResourceSetImpl resourceSet = new ReqIFResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("reqif", new ReqIFResourceFactoryImpl());

		return resourceSet;
	}

	private static void registerEPackageProRSpecific() {
		EPackage.Registry.INSTANCE.put(ConfigurationPackage.eNS_URI,
				ConfigurationPackage.eINSTANCE);
		EPackage.Registry.INSTANCE.put(HeadlinePackage.eNS_URI,
				HeadlinePackage.eINSTANCE);
	}

	private static void registerEPackageStd() {
		EPackage.Registry.INSTANCE.clear();
		EPackage.Registry.INSTANCE.put(ReqIF10Package.eNS_URI,
				ReqIF10Package.eINSTANCE);
		EPackage.Registry.INSTANCE.put(XhtmlPackage.eNS_URI,
				XhtmlPackage.eINSTANCE);
		EPackage.Registry.INSTANCE.put(DatatypesPackage.eNS_URI,
				DatatypesPackage.eINSTANCE);
		EPackage.Registry.INSTANCE.put(XMLNamespacePackage.eNS_URI,
				XMLNamespacePackage.eINSTANCE);
	}
}
